### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### This file includes the functions for collecting and saving ecological output data.
###

#XXX NOP functions while I try and get Makie to work
visualisemap(model) = nothing
populationtrends(model) = nothing

"""
    initecologicaldata()

Create output files for each data group collected by the nature model.
"""
function initecologicaldata(model::AgentBasedModel)
    newdataoutput!(model, "populations", ["Date", "Species", "Abundance"],
                   savepopulationdata, @param(nature.popoutfreq), populationtrends)
    newdataoutput!(model, "individuals", ["Date","ID","X","Y","Species","Sex","Age"],
                   saveindividualdata, @param(nature.indoutfreq), visualisemap)
end

"""
    savepopulationdata(model)

Return a comma-separated set of lines (to be printed to `populations.csv`), giving
the current date and population size for each animal species. May be called never,
daily, monthly, yearly, or at the end of a simulation, depending on the parameter
`nature.popoutfreq`.
"""
function savepopulationdata(model::AgentBasedModel)
    pops = Dict{String,Int}(s=>0 for s = @param(nature.targetspecies))
    for a in allagents(model)
        (typeof(a) != Animal) && continue
        pops[a.traits["name"]] += 1
    end
    data = []
    for p in keys(pops)
        push!(data, [model.date, p, pops[p]])
    end
    data
end

"""
    saveindividualdata(model)

Return a comma-separated set of lines (to be printed to `individuals.csv`), listing
all properties of all animal individuals in the model. May be called never, daily,
monthly, yearly, or at the end of a simulation, depending on the parameter
`nature.indoutfreq`. WARNING: Produces very big files!
"""
function saveindividualdata(model::AgentBasedModel)
    data = []
    for a in allagents(model)
        (typeof(a) != Animal) && continue
        push!(data, [model.date,a.id,a.pos[1],a.pos[2],a.traits["name"],a.sex,a.age])
    end
    data
end

